#ifndef GPIB_H_
#define GPIB_H_
#include <p18f4620.h>
//
//#define SOFTWARE_PP	// Define to use software-only handling of parallell poll
//
// Pin definitions. Note that there is stuff that needs chaning in the code to make it truly
// pin-portable. Look in gpib.c for "touch REN"
//
#define DIO_PORT	PORTD
#define	DIO_LAT		LATD
#define	DIO_TRIS	TRISD

#define DAV_PORT	PORTBbits.RB0
#define DAV_LAT		LATBbits.LATB0
#define DAV_TRIS	TRISBbits.RB0
#define NRFD_PORT	PORTBbits.RB1
#define NRFD_LAT	LATBbits.LATB1
#define NRFD_TRIS	TRISBbits.RB1
#define NDAC_PORT	PORTBbits.RB2
#define NDAC_LAT	LATBbits.LATB2
#define NDAC_TRIS	TRISBbits.RB2
#define REN_PORT	PORTBbits.RB3
#define REN_LAT		LATBbits.LATB3
#define REN_TRIS	TRISBbits.RB3
#define ATN_PORT	PORTBbits.RB4
#define ATN_LAT		LATBbits.LATB4
#define ATN_TRIS	TRISBbits.RB4
#define EOI_PORT	PORTBbits.RB5
#define EOI_LAT		LATBbits.LATB5
#define EOI_TRIS	TRISBbits.RB5
#define IFC_PORT	PORTBbits.RB6
#define IFC_LAT		LATBbits.LATB6
#define IFC_TRIS	TRISBbits.RB6
#define SRQ_PORT	PORTBbits.RB7
#define SRQ_LAT		LATBbits.LATB7
#define SRQ_TRIS	TRISBbits.RB7

#define PP_PORT		PORTAbits.RA0		// Port used to set response to parallell poll. 0=active, 1=inactive
#define PP_LAT		LATAbits.LATA0
#define PP_TRIS		TRISAbits.RA0

#define LED1_PORT		PORTAbits.RA1
#define LED1_LAT		LATAbits.LATA1
#define LED1_TRIS		TRISAbits.RA1
#define LED2_PORT		PORTAbits.RA2
#define LED2_LAT		LATAbits.LATA2
#define LED2_TRIS		TRISAbits.RA2

        // Chip Select Signal. Fix: These are hardcoded in mmc.c

#define SD_CS               LATCbits.LATC0
#define SD_CS_TRIS          TRISCbits.TRISC0

#define GPIB_S_DAT	1
#define GPIB_R_DAT	2
#define GPIB_R_SRQ	3
#define RD_AD_CMD 	4
#define SET_IO_CMD 	5
#define RESET_CMD       6

#define FLASH_SECTION_READ     0x01
#define FLASH_SECTION_WRITE    0x02

#define TIMEOUT 100000
#define true 1
#define false 0
//
// GPIB receive finite-state-machine states
//
#define GPIB_RX_START			0
#define GPIB_RX_ACCEPT			1
#define GPIB_RX_WAIT_DAV 		2
#define GPIB_RX_DAV_LOW			3
#define GPIB_RX_WAIT_DAV_HIGH 	4
#define GPIB_RX_DAV_HIGH 		5
#define GPIB_RX_EOI				6
#define GPIB_RX_FINISH 			9
#define GPIB_RX_DAV_TIMEOUT		10
#define GPIB_RX_DONE 			99

//
// GPIB send finite-state-machine states
//
#define GPIB_TX_START			0
#define GPIB_TX_CHECK			1
#define GPIB_TX_PUT_DATA		2
#define GPIB_TX_WAIT_FOR_NRFD	3
#define GPIB_TX_SET_DAV_LOW		4
#define GPIB_TX_WAIT_FOR_NDAC	5
#define GPIB_TX_SET_DAV_HIGH	6
#define GPIB_TX_FINISH 			9
#define GPIB_TX_ERROR 			98
#define GPIB_TX_DONE 			99

#define GPIB_MAX_TX_LEN			256		// Max length of transmit GPIB string
#define GPIB_MAX_RX_LEN			256		// Max length of receive GPIB string

#define MAX_DBG_STRING			40

#define EOIFLAG2 = 254;

extern unsigned char gpib_buff[GPIB_MAX_RX_LEN];
extern unsigned int gpib_ptr;
extern char gpib_tx_buff[GPIB_MAX_TX_LEN];
extern char gpib_tx_ptr;
extern unsigned char PP_BYTE;
extern unsigned char idbyte1;
extern unsigned char idbyte2;

void gpib_init(void);
void sendgpib (int len, int atn);
int readgpib(void);
void gpib_ren(unsigned char state);
void gpib_ifc(void);
void TimeGPIB(char state);
extern volatile char gpib_flags;
// Flag bit definitions
//
#define FLAGS_AUTO	1			// True if instrument sets to talk after command
#define FLAGS_EOI	2			// True in case we should set EOI on last char of command
#define FLAGS_EOT_ENABLE	4	// True in case we should add an EOT char after EOI
#define FLAGS_MODE	8			// True if we are controller
#define FLAGS_LON	16			// True if we are in listen only mode

#if defined(SOFTWARE_PP)
#warning Compiled for SOFTWARE PP
#define EnablePP()	PP_BYTE = 0x7f;PP_PORT = 1;	// Software Parallell Poll response
#define DisablePP()	PP_BYTE = 0xff;PP_PORT = 1;
#else
#warning Compiled for HARDWARE PP
#define EnablePP()	PP_PORT = 0;	// Hardware Parallell Poll response
#define DisablePP()	PP_PORT = 1;
#endif
#endif
