/*-------------------------------------------------------------------------
  sio.c - Serial in/out 

             (c) 2010 Anders Gustafsson <anders.gustafsson@pedago.fi> 

-------------------------------------------------------------------------*/

#include <stdio.h>
#include <p18f4620.h>
#include "sio.h"
#include "gpib.h"
#include <usart.h>

#define BAUD 38400						// Desired baud rate goes here

#define PIC_CLK 32000000 //MHz
//#define PIC_CLK 16000000 //MHz
#warning Compiled for 32.00Mhz and a baudrate of BAUD


volatile unsigned char rx_buff[128];		// Receive data buffer
volatile unsigned char rx_ptr;			// pointer to next slot in receive buffer
volatile union
{
	struct
	{
		unsigned DataAvailable:1;
		unsigned :7;
	};
} rx_flagsbits;

int cnt;

//#pragma code rx_interrupt = 0x08
//void rx_int(void)
//{
//	_asm goto rx_handler _endasm
//}
//#pragma code
//#pragma interrupt rx_handler
//void rx_handler(void)
//{
//	unsigned char c;
//
//	c = RCREG;
//	if(rx_ptr < 128 && c!=0x03)			// Simplistic receive buffering. Stuff data into buffer, excluding out EOT
//	{									// and raise flag when we have received the entire string
//		rx_buff[rx_ptr] = c;
//		++rx_ptr;
//		//
//		
//
//	}
//	if( c == 0x0a || c == 0x03 || rx_ptr >= 128-1) rx_flagsbits.DataAvailable = 1; // LF or Ctrl-C terminates message
//}
void sio_setup(void)
{

	//#define SPBRG_VALUE ((PIC_CLK/(16UL * BAUD) -1))
	//#define SPBRG_VALUE ((PIC_CLK/(16UL * BAUD) -1))
	#define SPBRG_VALUE ((PIC_CLK/(64UL * BAUD) -1))
	#define BRGH_VALUE ((PIC_CLK/(64UL * BAUD) -1))>>8


	SPBRG=SPBRG_VALUE;
	SPBRGH=BRGH_VALUE;	//data rate for sending (see 16F876 manual under 'USART') 

	BAUDCONbits.BRG16 = 0;
	TXSTAbits.SYNC=0;						//asynchronous
	RCSTAbits.SPEN=1;						//enable serial port pins
	RCSTAbits.CREN=1;						//enable reception
	RCSTAbits.SREN=0;						//no effect
	PIE1bits.TXIE=0;						//disable tx interrupts
	PIE1bits.RCIE=0;						//disable rx interrupts
	RCONbits.IPEN=1;						//enable interrupt priority
	IPR1bits.RCIP=1;						//make receive interrupt high priority
	INTCONbits.GIEH=1;						//enable all high priority interrupts
	TXSTAbits.TX9D=0;						//8-bit transmission
	RCSTAbits.RX9=0;						//8-bit reception
	TXSTAbits.TXEN=0;						//reset transmitter
	TXSTAbits.TXEN=1;						//enable the transmitter
	TXSTAbits.BRGH=0;
	//
	rx_flagsbits.DataAvailable = 0;
	rx_ptr=0;
}

//writes a character to the serial port
void sio_putch(unsigned char c)
{
//	while(!PIR1bits.TXIF)			//set when register is empty
//	{
//		clear_usart_errors_inline();
//		ClrWdt();
//	}
//	TXREG=c;
	while(BusyUSART());
	putcUSART(c);
}

//gets a character from the serial port without timeout
unsigned char sio_getch(void)
{
	while(!PIR1bits.RCIF)
	{
		ClrWdt();
		clear_usart_errors_inline();
	}
	return RCREG;
}

unsigned char sio_getch_available(void)
{
  if (PIR1bits.RCIF) {
    return true;
  }
  return false;
}

void clear_usart_errors_inline(void)
{
	unsigned char dummy;
	if (RCSTAbits.OERR)			// Overrun error											
	{																	
		TXSTAbits.TXEN=0;													
		TXSTAbits.TXEN=1;													
		RCSTAbits.CREN=0;													
		RCSTAbits.CREN=1;												
	}																	
	if (RCSTAbits.FERR)			// Framing error										
	{																	
		dummy=RCREG;										
		TXSTAbits.TXEN=0;													
		TXSTAbits.TXEN=1;													
	}
}

void sio_puts(unsigned char *s)
{
	//putsUSART(s);
	cnt=0;
	while(*s != 0)
	{
		sio_putch(*s);
		++s;
		++cnt;
	}
	if(cnt>=MAX_DBG_STRING)
	{
		sio_putch('#');
		sio_putch('O');
		sio_putch('F');
		sio_putch('#');
	}
}